***
      PROGRAM colour
c-------------------------------------------------------------c
c
c     Example program to show how to compute Johnson colours using 
c     Kurucz and Bergeron (the latter for the white dwarfs) 
c     model atmosphere data. 
c     U, B, V, R and I are available. 
c     Requires data files: 
c         Kurucz.dat
c         (Kurucz, 1992, Proc. IAU Symp. 149, p. 225)
c         wdhyd.dat
c         (Bergeron, Wesemael & Beauchamp, 1995, PASP, 107, 1047)
c
c     Also shown are examples for calculating HST/ACS colours. 
c     F435W, F475W, F555W, F606W, F625W, F775W and F814W are available 
c     (although only F435W, F555W, F606W, F814W for white dwarfs). 
c     Requires data files: 
c         acscols.dat  
c         berg_da_hst.dat
c         (Castelli & Kurucz, 2003, Proc. IAU Symp. 210)
c
c     User needs to set the metallicity. 
c     Given Mass, Luminosity and Radius of a star the colours 
c     can then be computed. 
c
c     Written by Jarrod Hurley 10/08/06. 
c
c-------------------------------------------------------------c
      implicit none
*
      INTEGER kw
*
      REAL*8 z,massi,logl,logr,logte,logz,bolc
      REAL*8 mv,mb,mi,bminv,vmini,uminb
      REAL*8 mv1,mv2,bminv1,bminv2
      LOGICAL acscol
      CHARACTER *100 HEAVYMETAL,KWin,MASSin
      CHARACTER *100 LOGLin,LOGRin

      CALL GETARG(1,HEAVYMETAL)
      CALL GETARG(2,KWin)
      CALL GETARG(3,MASSin)
      CALL GETARG(4,LOGLin)
      CALL GETARG(5,LOGRin)

*
* Set this true or false depending on whether or not you want ACS 
* colours. 
*
      acscol = .false.
*
* Set metallicity. 
*
*      z = 0.001d0
      READ(HEAVYMETAL,*) z
      logz = log10(z/0.02)
*
* Read in the bolometric correction tables. 
*
      CALL ubvtab
*
* Mass, luminosity and radius of a star (as well as metallicity) 
* are required to compute colours. 
* This example code also uses the stellar index from SSE 
* (see Hurley, Pols & Tout 2000, MNRAS, 315, 543), simply to 
* distinguish between regular stars (kw = 1 denotes a main-sequence 
* star, for example) and white dwarfs (kw values of 10, 11 or 12). 
*
* Here are some test values: 
*
* 2 Msun turn-off star, Z = 0.001

      kw = 1
      READ(KWin,*) kw
      massi = 2.0
      READ(MASSin,*) massi
      logl = 1.8355
      READ(LOGLin,*) logl
      logr = 0.3955
      READ(LOGRin,*) logr
*     massi = 0.8
*     logl = 0.2091
*     logr = 0.0426
*     massi = 0.75
*     logl = -0.0976
*     logr = -0.071
*     massi = 0.61
*     logl = 3.3684
*     logr = 2.0307
* 1.3 Msun turn-off star, Z = 0.02
*     massi = 1.3 
*     logl = 0.7574    
*     logr = 0.3369
* 0.55 Msun cool WD 
*     kw = 11
*     massi = 0.55
*     logl = -3.1694   
*     logr = -1.8712
*
      logte = 3.762d0 + 0.25d0*logl - 0.5d0*logr
      if(acscol)then
         if(kw.ge.10.and.kw.le.12)then
            CALL wd2acs(logl,logte,massi,bolc,mv,
     &                  mb,mv1,mi)
         else
            CALL lt2acs(logl,logte,massi,logz,bolc,mv,
     &                  mb,mv1,mi)
         endif
         bminv = mb - mv1
         vmini = mv1 - mi
      else
         if(kw.ge.10.and.kw.le.12)then
            CALL wd2ubv(logl,logte,massi,bolc,mv1,
     &                  uminb,bminv,vmini)
         else
            CALL lt2ubv(logl,logte,massi,logz,bolc,mv1,
     &                  uminb,bminv,vmini)
         endif
         mb = mv1 + bminv
      endif
*
      WRITE(*,*)
      WRITE(*,*)' Single Star:'
      if(acscol)then
         WRITE(*,*)'           F606W ',mv1
         WRITE(*,*)'           F555W ',mb
         WRITE(*,*)'     F555W-F606W ',bminv
         WRITE(*,*)'     F606W-F814W ',vmini
      else
         WRITE(*,*)'              Mv ',mv1
         WRITE(*,*)'              Mb ',mb
         WRITE(*,*)'             B-V ',bminv
         WRITE(*,*)'             V-I ',vmini
      endif
*
* Here is an example for an unresolved binary. 
*
      bminv1 = bminv
*
* Take a 0.55 Msun WD companion. 
      kw = 11
      massi = 0.55
      logl = -3.1694   
      logr = -1.8712
*
      logte = 3.762d0 + 0.25d0*logl - 0.5d0*logr
      if(acscol)then
         if(kw.ge.10.and.kw.le.12)then
            CALL wd2acs(logl,logte,massi,bolc,mv,
     &                  mb,mv2,mi)
         else
            CALL lt2acs(logl,logte,massi,logz,bolc,mv,
     &                  mb,mv2,mi)
         endif
         bminv2 = mb - mv2
      else
         if(kw.ge.10.and.kw.le.12)then
            CALL wd2ubv(logl,logte,massi,bolc,mv2,
     &                  uminb,bminv2,vmini)
         else
            CALL lt2ubv(logl,logte,massi,logz,bolc,mv2,
     &                  uminb,bminv2,vmini)
         endif
      endif
*
* Add fluxes to get combined magnitudes. 
      mv = -2.5d0*log10(10.d0**(-0.4d0*mv1) + 10.d0**(-0.4d0*mv2))
      mb = -2.5d0*log10(10.d0**(-0.4d0*(mv1+bminv1)) +
     &                  10.d0**(-0.4d0*(mv2+bminv2)))
      bminv = mb - mv
*
*      WRITE(*,*)
*      WRITE(*,*)' Binary Star:'
*      if(acscol)then
*         WRITE(*,*)'           F606W ',mv
*         WRITE(*,*)'           F555W ',mb
*         WRITE(*,*)'     F555W-F606W ',bminv
*      else
*         WRITE(*,*)'              Mv ',mv
*         WRITE(*,*)'              Mb ',mb
*         WRITE(*,*)'             B-V ',bminv
*      endif
*
      STOP
      END
***
      SUBROUTINE ubvtab
      implicit none
*
      INTEGER i,j,k,n
      INTEGER nzgr,ntgr,nggr
      PARAMETER(nzgr=8,ntgr=61,nggr= 11)
      INTEGER ntgr2,nggr2
      PARAMETER(ntgr2=91,nggr2=5)
      INTEGER nzgr3,ntgr3,nggr3
      PARAMETER(nzgr3=9,ntgr3=76,nggr3= 11)
      INTEGER ntgr4,nggr4
      PARAMETER(ntgr4=57,nggr4=5)
*
      REAL feh
      REAL*8 mbol,mv,ms
      REAL*8 zgr(nzgr),tgr(ntgr),ggr(nggr),ubv(nzgr,ntgr,nggr,5)
      COMMON /ubvdata/ zgr,tgr,ggr,ubv
      REAL*8 wtgr(ntgr2),wggr(nggr2),wubv(ntgr2,nggr2,5)
      COMMON /wubvdata/ wtgr,wggr,wubv
      REAL*8 azgr(nzgr3),atgr(ntgr3),aggr(nggr3)
      REAL*8 aubv(nzgr3,ntgr3,nggr3,10)
      COMMON /acsdata/ azgr,atgr,aggr,aubv
      REAL*8 awtgr(ntgr4),awggr(nggr4),awubv(ntgr4,nggr4,7)
      COMMON /wacsdata/ awtgr,awggr,awubv
*
      OPEN(21,file='Kurucz.dat',
     &        form='formatted',status='old')
      do k = 1, nzgr
         do i = 1, ntgr
            do j = 1, nggr
               read(21,*)feh,tgr(i),ggr(j),(ubv(k,i,j,n),n=1,5)
            end do
            tgr(i) = log10(tgr(i))
         end do
c....... zgr=log(Z/0.02), assuming X=0.76-3*Z and Z(sun)=0.02
         zgr(k) = -log10((3.d0 + 37.425d0*10.d0**(-feh))/38.d0)
*        zgr(k) = -log10(0.07895 + 0.92105*10.0**(-feh))
      end do
      CLOSE(21)
*
      OPEN(22,file='wdhyd.dat',
     &        form='formatted',status='old')
      do j = 1,nggr2
         do i = 1,ntgr2
            read(22,*)wtgr(i),wggr(j),(wubv(i,j,k),k=1,5)
         enddo
      enddo
      do i = 1,ntgr2
         wtgr(i) = log10(wtgr(i))
      enddo
      CLOSE(22)
*
      OPEN(23,file='acscols.dat',
     &        form='formatted',status='old')
      do k = 1, nzgr3
         do i = 1, ntgr3
            do j = 1, nggr3
               read(23,*)feh,atgr(i),aggr(j),mbol,(aubv(k,i,j,n),n=1,10)
            end do
            atgr(i) = log10(atgr(i))
         end do
c....... zgr=log(Z/0.02), assuming X=0.76-3*Z and Z(sun)=0.02
         azgr(k) = -log10((3.0 + 37.425*10.0**(-feh))/38.0)
*        azgr(k) = -log10(0.07895 + 0.92105*10.0**(-feh))
      enddo
      CLOSE(23)
*
      OPEN(24,file='berg_da_hst.dat',
     &        form='formatted',status='old')
      do j = 1,nggr4
         do i = 1,ntgr4
            read(24,*)awtgr(i),awggr(j),awubv(i,j,1),ms,
     &                awubv(i,j,2),mv,(awubv(i,j,k),k=3,7),feh
         enddo
      enddo
      do i = 1,ntgr4
         awtgr(i) = log10(awtgr(i))
      enddo
      CLOSE(24)
*
      RETURN
      END
***
      subroutine lt2ubv(logl,logt,mass,logz,bolc,mv,uminb,bminv,vmini)
      implicit none
c.... computes values of Mv, U-B, B-V and V-I for given log L, log T,
c.... mass and log(Z/0.02)
*
      integer k,ig,it,iz
      integer nzgr,ntgr,nggr
      parameter(nzgr=8,ntgr=61,nggr=11)
      integer indx
      external indx
      real*8 zgr(nzgr),tgr(ntgr),ggr(nggr),ubv(nzgr,ntgr,nggr,5)
      common /ubvdata/ zgr,tgr,ggr,ubv
      real*8 cm(5),gconst
      parameter(gconst=-10.6071d0)
      real*8 logl,logt,mass,logz,mv,uminb,bminv,vmini
      real*8 logm,logg,dg1,dg2,dt1,dt2,dz1,dz2,mbol,bolc
*
      logm = dlog10(mass)
      logg = logm + 4.d0*logt - logl + gconst
c.... find indices of log Z, log g and log T to interpolate between.
c.... don't allow extrapolation outside log Z and log g grid.
      ig = indx(logg,ggr,nggr)
      it = indx(logt,tgr,ntgr)
      iz = indx(logz,zgr,nzgr)
      dg1 = (logg - ggr(ig-1))/(ggr(ig) - ggr(ig-1))
      dg1 = max(0.d0, min(1.d0, dg1))
      dg2 = 1.d0 - dg1
      dt1 = (logt - tgr(it-1))/(tgr(it) - tgr(it-1))
      dt2 = 1.d0 - dt1
      dz1 = (logz - zgr(iz-1))/(zgr(iz) - zgr(iz-1))
      dz1 = max(0.d0, min(1.d0, dz1))
      dz2 = 1.d0 - dz1
      do k = 1, 5
         cm(k) = ((ubv(iz,it,ig,k)*dg1 + ubv(iz,it,ig-1,k)*dg2)*dt1
     &            + (ubv(iz,it-1,ig,k)*dg1 +
     &            ubv(iz,it-1,ig-1,k)*dg2)*dt2)*dz1 +
     &           ((ubv(iz-1,it,ig,k)*dg1 +
     &            ubv(iz-1,it,ig-1,k)*dg2)*dt1 +
     &           (ubv(iz-1,it-1,ig,k)*dg1 +
     &            ubv(iz-1,it-1,ig-1,k)*dg2)*dt2)*dz2
      enddo
      mbol = 4.75d0 - 2.5d0*logl
      bolc = cm(1)
      mv = mbol - bolc
      uminb = cm(2)
      bminv = cm(3)
      vmini = cm(4) + cm(5)
*
      return
      end
***
      subroutine lt2acs(logl,logt,mass,logz,bolc,mv,m555w,m606w,m814w)
      implicit none
c.... computes values of Mv, BC, V, V-F435W, V-F475W, etc. in HST ACS system
c.... for given log L, log T, mass and log(Z/0.02)
*
      integer k,ig,it,iz
      integer nzgr3,ntgr3,nggr3
      parameter(nzgr3=9,ntgr3=76,nggr3=11)
      integer indx
      external indx
      real*8 azgr(nzgr3),atgr(ntgr3),aggr(nggr3)
      real*8 aubv(nzgr3,ntgr3,nggr3,10)
      common /acsdata/ azgr,atgr,aggr,aubv
      real*8 cm(10),gconst
      parameter(gconst=-10.6071)
      real*8 logl,logt,mass,logz,mv,uminb,bminv,vmini
      real*8 m435w,m475w,m555w,m606w,m625w,m775w,m814w
      real*8 logm,logg,dg1,dg2,dt1,dt2,dz1,dz2,mbol,bolc
*
      logm = log10(mass)
      logg = logm + 4*logt - logl + gconst
c.... find indices of log Z, log g and log T to interpolate between.
c.... don't allow extrapolation outside log Z and log g grid.
      ig = indx(logg,aggr,nggr3)
      it = indx(logt,atgr,ntgr3)
      iz = indx(logz,azgr,nzgr3)
      dg1 = (logg - aggr(ig-1))/(aggr(ig) - aggr(ig-1))
      dg1 = max(0.0, min(1.0, dg1))
      dg2 = 1.0 - dg1
      dt1 = (logt - atgr(it-1))/(atgr(it) - atgr(it-1))
      dt2 = 1.0 - dt1
      dz1 = (logz - azgr(iz-1))/(azgr(iz) - azgr(iz-1))
      dz1 = max(0.0, min(1.0, dz1))
      dz2 = 1.0 - dz1
      do k = 1,10
         cm(k) = ((aubv(iz,it,ig,k)*dg1 + aubv(iz,it,ig-1,k)*dg2)*dt1
     &            + (aubv(iz,it-1,ig,k)*dg1 +
     &            aubv(iz,it-1,ig-1,k)*dg2)*dt2)*dz1 +
     &           ((aubv(iz-1,it,ig,k)*dg1 +
     &            aubv(iz-1,it,ig-1,k)*dg2)*dt1 +
     &           (aubv(iz-1,it-1,ig,k)*dg1 +
     &            aubv(iz-1,it-1,ig-1,k)*dg2)*dt2)*dz2
      enddo
      mbol = 4.75 - 2.5*logl
      bolc = cm(2)
*     bolc = cm(1)
      mv = mbol - bolc
      m435w = mv - cm(4)
      m475w = mv - cm(5)
      m555w = mv - cm(6)
      m606w = mv - cm(7)
      m625w = mv - cm(8)
      m775w = mv - cm(9)
      m814w = mv - cm(10)
      uminb = cm(6) - cm(4)
      bminv = cm(7) - cm(6)
      vmini = cm(10) - cm(7)
*
      return
      end
***
      subroutine wd2acs(logl,logt,mass,bolc,mv,m555w,m606w,m814w)
c.... computes values of Mbol, BC, M336, M439, M555, M606, M814
c.... in HST ACS system for given logL, logT and mass.
      implicit none
      integer k,ig,it
      integer ntgr4,nggr4
      parameter(ntgr4=57,nggr4=5)
      integer indx
      external indx
      REAL*8 awtgr(ntgr4),awggr(nggr4),awubv(ntgr4,nggr4,7)
      COMMON /wacsdata/ awtgr,awggr,awubv
      real*8 cm(7),gconst
      parameter(gconst = -10.6071)
      real*8 logl,logt,mass,mv,uminb,bminv,vmini
      real*8 logm,logg,dg1,dg2,dt1,dt2,mbol,bolc
      real*8 m439w,m555w,m606w,m814w
*
      logm = log10(mass)
      logg = logm + 4*logt - logl + gconst
c.... find indices of log g and log T to interpolate between.
c.... don't allow extrapolation outside log g grid.
      ig = indx(logg, awggr, nggr4)
      it = indx(logt, awtgr, ntgr4)
      dg1 = (logg - awggr(ig-1))/(awggr(ig) - awggr(ig-1))
      dg1 = max(0.0, min(1.0, dg1))
      dg2 = 1.0 - dg1
      dt1 = (logt - awtgr(it-1))/(awtgr(it) - awtgr(it-1))
      dt2 = 1.0 - dt1
      do k = 1,7
         cm(k) = (awubv(it,ig,k)*dg1 + awubv(it,ig-1,k)*dg2)*dt1
     &         + (awubv(it-1,ig,k)*dg1 + awubv(it-1,ig-1,k)*dg2)*dt2
      enddo
      mbol = 4.75 - 2.5*logl
      bolc = cm(1)
      mv = mbol - bolc
      m439w = cm(4)
      m555w = cm(5)
      m606w = cm(6)
      m814w = cm(7)
      end
***
      integer function indx(ax,xx,nx)
c.....finds index of ax in monotonously increasing or decreasing array xx
      implicit none
      integer nx,j,jl,jh
      real*8 ax,xx(nx),sx
*
      sx = xx(nx) - xx(1)
      jl = 1
      jh = nx
 1    if (jh-jl.gt.1) then
         j = (jh + jl)/2
         if ((ax-xx(j))*sx.gt.0.d0) then
            jl = j
         else
            jh = j
         end if
         goto 1
      end if
      indx = jh
*
      return
      end
***
      subroutine wd2ubv(logl,logt,mass,bolc,mv,uminb,bminv,vmini)
c.... computes values of Mv, U-B, B-V and V-I for given log L, log T,
c.... mass and log(Z/0.02)
      implicit none
      integer k,ig,it
      integer ntgr,nggr
      parameter(ntgr=91,nggr=5)
      integer indx
      external indx
      real*8 tgr(ntgr),ggr(nggr),ubv(ntgr,nggr,5)
      common /wubvdata/ tgr,ggr,ubv
      real*8 cm(5),gconst
      parameter(gconst = -10.6071d0)
      real*8 mbol,bolc,mv,uminb,bminv,vmini
      real*8 logm,logg,dg1,dg2,dt1,dt2,mass,logt,logl
*
      logm = dlog10(mass)
      logg = logm + 4.d0*logt - logl + gconst
c.... find indices of log g and log T to interpolate between.
c.... don't allow extrapolation outside log g grid.
      ig = indx(logg,ggr,nggr)
      it = indx(logt,tgr,ntgr)
      dg1 = (logg - ggr(ig-1))/(ggr(ig) - ggr(ig-1))
      dg1 = MAX(0.d0,MIN(1.d0,dg1))
      dg2 = 1.d0 - dg1
      dt1 = (logt - tgr(it-1))/(tgr(it) - tgr(it-1))
      dt2 = 1.d0 - dt1
      do k = 1,5
         cm(k) = (ubv(it,ig,k)*dg1 + ubv(it,ig-1,k)*dg2)*dt1
     &         + (ubv(it-1,ig,k)*dg1 + ubv(it-1,ig-1,k)*dg2)*dt2
      enddo
      mbol = 4.75d0 - 2.5d0*logl
      bolc = cm(1)
      mv = mbol - bolc
      uminb = cm(2)
      bminv = cm(3)
      vmini = cm(4) + cm(5)
      return
      end
***
